/* Copyright (C) 2001  Sony Computer Entertainment Inc.
 *
 * "basic.vcl"
 *
 */
.include 	"vumacros.m"

; ---------------------------------------------------------------------
;
;
;       vu1 micro code for basic sample
;
;
;	Transform0 - Transform3 	: perspective + transformation matrix 
;	LightMatrix0 - LightMatrix3 	: light color matrix
;       LightColor0 - LightColor3     :  X light vector matrix(3 lighting vector + 1 ambient)
;	GifTag        	: gif tag 
;	vert_in 		: source vertex
;	normal 		: source normal
;	ST 		: source st
;	color 		: source color
;
;
;	iKick : pointer for gif tag / for XGKICK
;	iVert_in : pointer for vertex
;	iNorm : pointer for normal
;	iST_in : pointer for st
;	iColor_in : pointer for color
;	iRGBAQ_out : store pointer for RGBAQ
;	iST_out : store pointer for ST
;	iXYZF2_out : store pointer for XYZF2
;
;       iVert_count : counter for vertices
;
;
; ---------------------------------------------------------------------

kScreenMat0		.assign		0
kScreenMat1		.assign		1
kScreenMat2		.assign		2
kScreenMat3		.assign		3
kLocalWorld0		.assign		4
kLocalWorld1		.assign		5
kLocalWorld2		.assign		6
kLocalWorld3		.assign		7
kLightColor0		.assign		8
kLightColor1		.assign		9
kLightColor2		.assign		10
kLightColor3		.assign		11
kLightMatrix0		.assign		12
kLightMatrix1		.assign		13
kLightMatrix2		.assign		14
kLightMatrix3		.assign		15
;kTransform0		.assign		16
;kTransform1		.assign		17
;kTransform2		.assign		18
;kTransform3		.assign		19


        .init_vf_all    ; allow vcl to use all VF registers
        .init_vi_all    ; allow vcl to use all VI registers
        .syntax new
 
        .vu
 
        --enter         ; declare an entry point to the code
                        ; there are no externally set registers..
        --endenter      ; end declaration of entry point

	lq ScreenMat[0], kScreenMat0(vi00); (0) load screen matrix
	lq ScreenMat[1], kScreenMat1(vi00)
	lq ScreenMat[2], kScreenMat2(vi00)
	lq ScreenMat[3], kScreenMat3(vi00)
	lq LocalWorld[0], kLocalWorld0(vi00); (4) load local world matrix
	lq LocalWorld[1], kLocalWorld1(vi00)
	lq LocalWorld[2], kLocalWorld2(vi00)
	lq LocalWorld[3], kLocalWorld3(vi00)

	mat_x_mat Transform, LocalWorld, ScreenMat	; mat_x_mat nm,lm,rm   nm = lm x rm

	lq LightColor[0], kLightColor0(vi00); (8) load light color matrix 
	lq LightColor[1], kLightColor1(vi00)
	lq LightColor[2], kLightColor2(vi00)
	lq LightColor[3], kLightColor3(vi00)
	lq LightMatrix[0], kLightMatrix0(vi00)	; (12) load light vector matrix 
	lq LightMatrix[1], kLightMatrix1(vi00)
	lq LightMatrix[2], kLightMatrix2(vi00)
	lq LightMatrix[3], kLightMatrix3(vi00)
	;lq Transform[0], kTransform0(vi00)	; (16) load screen x transformation matrix
	;lq Transform[1], kTransform1(vi00)
	;lq Transform[2], kTransform2(vi00)
	;lq Transform[3], kTransform3(vi00)
	sub.z Offset,vf0,vf0
	loi -20000.0 
	addi.z Offset2,vf0,i

	--cont

START0:
	xtop iBase
	ilwr.x iVert_count, (iBase)x 		; copy to iVert_count
	iaddiu iTmp, vi00, 0x7fff	; set mask
	iand iVert_count, iTmp, iVert_count		; mask. iVert_count is counter for vertices.
	lq GifTag, 0(iBase) 		; load gif tag
	iaddiu iVert_in, iBase, 1		; pointer for vertices
	iadd iNorm, iVert_in, iVert_count		; pointer for normal
	iadd iColor_in, iNorm, iVert_count		; pointer for color
	iadd iST_in, iColor_in, iVert_count		; pointer for st
	iadd iKick, iST_in, iVert_count		; set XGKICK pointer
	sq GifTag, 0(iKick) 		; store gif tag
	iaddiu iRGBAQ_out, iKick, 2		; set store pointer for RGBAQ
	iaddiu iST_out, iKick, 1		; set store pointer for ST
	iaddiu iXYZF2_out, iKick, 3		; set store pointer for XYZF2

LOOP:
	--LoopCS 6,6			; tell VCL that we loop at least 6 times
	lqi vert_in, (iVert_in++)		; load vertex
	lqi normal, (iVert_in++)		; load normal
	lqi color, (iVert_in++)		; load color
	lqi ST, (iVert_in++)		; load st

	; --- rotation/translation/perspective transform ---
	vec_x_mat trans_vert, vert_in, Transform

	div q, vf00[w], trans_vert[w]

	; --- light matrix X normal ---
	vec_x_mat light_vec, normal, LightMatrix


	; --- calc transformed position and store it ---
	mulq floatScreenVec, trans_vert, q	
	add.z floatScreenVec, floatScreenVec,Offset
	ftoi4 ScreenVec, floatScreenVec	
	sq ScreenVec, 0(iXYZF2_out) ; --- store XYZF2 ---
	iaddiu iXYZF2_out, iXYZF2_out, 3

	; --- load st and store it ---
	mulq persST, ST, q
	sq persST, 0(iST_out)
	iaddiu iST_out, iST_out, 3

	; --- color * dot-product ---
	vec_x_mat light_col, light_vec, LightColor

	mul floatCol, color, light_col
	ftoi0 screenCol, floatCol
	sq screenCol, 0(iRGBAQ_out)
	iaddiu iRGBAQ_out, iRGBAQ_out, 3
	
	; --- decrement loop ---
	iaddi iVert_count, iVert_count, -1	
	ibne iVert_count, vi00, LOOP

	sub.z Offset,Offset2,Offset

	; --- send result to GIF and stop ---
	xgkick iKick  

	--cont

	b START0  

.END
